#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#------------------------------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
# IT WAS CREATED AND TESTED IN A WINDOWS ENVIRONMENT ONLY AND MAY NOT FUNCTION ON A MAC.
#------------------------------------------------------------------------------------------------
#    Name:	Birds Mouth Rafter
#      By:	sdmitch
#   Usage:	Create a template showing the various cuts needed to produce a birds mouth rafter or
#						a finished rafter with the cuts applied.
#		 Note:	
#			
#    Date:	Nov 2011
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
#
if not file_loaded?(File.basename(__FILE__))
    UI.menu('Plugins').add_item('Bird Mouth Rafter') { SDM_BMR.bird_mouth_rafter }
	 file_loaded(File.basename(__FILE__))
end
#
module SDM_BMR

	def self.bird_mouth_rafter
		mod = Sketchup.active_model
		ent = mod.entities
		sel = mod.selection
		@mat="2X10" if !@mat
		@slp="3:1" if !@slp
		@spn=8.feet if !@spn;# 2438.4mm
		@ovh=1.feet if !@ovh;# 304.8mm
		prompts = ["          Material:","Slope(h:v|deg):","      Total Span:","       Overhang:"]
		defaults = [@mat,@slp,@spn,@ovh]
		options = ["2X6|2X8|2X10|2X12"]
		inp = UI.inputbox(prompts,defaults,options,"Birds Mouth Rafter")
		if inp then
			@mat=inp[0];@slp=inp[1]; @spn=inp[2].to_l;@ovh=inp[3].to_l
			thickness=1.50.inch;# 38mm
			case @mat
				when "2X6"   : width = 5.50.inch;		#1 1/2 in  5 1/2 in (38mm  140mm)
				when "2X8"   : width = 7.25.inch;		#1 1/2 in  7 1/4 in (38mm  184mm)
				when "2X10" : width = 9.25.inch;		#1 1/2 in  9 1/4 in (38mm  235mm)
				when "2X12" : width = 11.25.inch;	#1 1/2 in  11 1/4 in (38mm  286mm)
			end
			begin
				h,v=inp[1].split(':')
				hor=eval(h).to_f;vert=eval(v).to_f;slope=Math.atan2(vert,hor);#puts "slope=#{slope}"
			rescue
				slope=inp[1].to_f.degrees;vert = 1.0; hor = vert/Math.tan(slope)
			end
			diagonal_cut=width/Math.cos(slope);#puts "diagonal_cut=#{diagonal_cut}"
			toe=diagonal_cut-@ovh*Math.tan(slope);#puts "toe=#{toe}"
			if toe < 0.0
				@ovh=diagonal_cut/Math.tan(slope)
				toe=0.0;UI.messagebox "Max overhang is #{@ovh.to_l}"
			end
			tail=@ovh*Math.cos(slope);waste=toe*Math.sin(slope)
			depth_of_cut=(width*0.25)/Math.cos(slope)
			set_cut=depth_of_cut/Math.sin(slope)
			hgt=@spn/hor;hyp=Math.sqrt(@spn**2 + hgt**2)/2.0
			p0=ORIGIN; pt=p0.offset(Z_AXIS,diagonal_cut-depth_of_cut)
			vec1=Geom::Vector3d.new(0,hor,vert).normalize
			vec2=Geom::Vector3d.new(0,vert,-hor).normalize
			vec3=Geom::Vector3d.new(0,vert,-hor).normalize
			mod.start_operation("Make Rafter")
			grp=ent.add_group; 
			#Define the cuts
			cp1=p0.offset(Z_AXIS.reverse,depth_of_cut)
			cp2=cp1.offset(Y_AXIS.reverse,@ovh)
			cp3=cp2.offset(Z_AXIS,toe)
			@len=(hyp + cp3.distance(pt) - 0.75.inch/Math.cos(slope)).to_l;# 19mm
			cp4=cp3.offset(vec1,@len)
			cp5=cp4.offset(Z_AXIS.reverse,diagonal_cut)
			cp6=p0.offset(Y_AXIS,set_cut)
			#Define the board
			mp1=p0.offset(Z_AXIS.reverse,depth_of_cut)
			mp2=mp1.offset(vec1.reverse,tail)
			mp3=mp2.offset(vec2.reverse,width)
			mp4=mp3.offset(vec1,@len+waste)
			mp5=mp4.offset(vec2,width)
			if UI.messagebox("Show cut lines only?",MB_YESNO)==6
				face=grp.entities.add_face(mp2,mp3,mp4,mp5)
				face.pushpull thickness
				grp.entities.add_cline(p0,cp6)
				grp.entities.add_cline(p0,cp1)
				grp.entities.add_cline(cp1,cp2)
				grp.entities.add_cline(cp2,cp3)
				grp.entities.add_cline(cp4,cp5)
				#Add dimensions
				vec1.length=vec2.length=width/4.0;vec3.length=width/2.5
				mp=Geom.linear_combination(0.5,p0,0.5,cp1)
				grp.entities.add_text(depth_of_cut.to_l.to_s,mp,vec3)
				mp=Geom.linear_combination(0.5,mp1,0.5,mp2)
				grp.entities.add_text(tail.to_l.to_s,mp,vec2)
				dist=mp2.distance cp2;mp=Geom.linear_combination(0.5,mp2,0.5,cp2)
				grp.entities.add_text(dist.to_l.to_s,mp,vec1.reverse)
				dist=mp3.distance cp3;mp=Geom.linear_combination(0.5,mp3,0.5,cp3)
				grp.entities.add_text(dist.to_l.to_s,mp,vec2.reverse)
				mp=Geom.linear_combination(0.5,cp3,0.5,mp4)
				grp.entities.add_text(@len.to_s,mp,vec2.reverse)
				dist=mp5.distance cp5;mp=Geom.linear_combination(0.5,mp5,0.5,cp5)
				grp.entities.add_text(dist.to_l.to_s,mp,vec2)
				dist=cp5.distance cp6;mp=Geom.linear_combination(0.5,cp5,0.5,cp6)
				grp.entities.add_text(dist.to_l.to_s,mp,vec2)
				mp=Geom.linear_combination(0.5,p0,0.5,cp6)
				grp.entities.add_text(set_cut.to_l.to_s,mp,vec2)
				dist=cp1.distance cp6;mp=Geom.linear_combination(0.5,cp1,0.5,cp6)
				grp.entities.add_text(dist.to_l.to_s,mp,vec2)
			else
				face=grp.entities.add_face(p0,cp1,cp2,cp3,cp4,cp5,cp6)
				face.pushpull thickness
			end
			mod.commit_operation
		else
			Sketchup.send_action "selectSelectionTool:"
		end
	end
end